@@ -80,7 +80,6 @@ static void gst_alsasink_init_interfaces (GType type);
 GST_BOILERPLATE_FULL (GstAlsaSink, gst_alsasink, GstAudioSink,
     GST_TYPE_AUDIO_SINK, gst_alsasink_init_interfaces);
 
-static void gst_alsasink_dispose (GObject * object);
 static void gst_alsasink_finalise (GObject * object);
 static void gst_alsasink_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -133,12 +132,6 @@ static GstStaticPadTemplate alsasink_sink_factory =
         "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, MAX ]")
     );
 
-static void
-gst_alsasink_dispose (GObject * object)
-{
-  G_OBJECT_CLASS (parent_class)->dispose (object);
-}
-
 static void
 gst_alsasink_finalise (GObject * object)
 {
@@ -191,7 +184,6 @@ gst_alsasink_class_init (GstAlsaSinkClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_alsasink_dispose);
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_alsasink_finalise);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_alsasink_get_property);
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_alsasink_set_property);
@@ -253,16 +245,9 @@ gst_alsasink_get_property (GObject * object, guint prop_id,
       g_value_set_string (value, sink->device);
       break;
     case PROP_DEVICE_NAME:
-      if (sink->handle) {
-        snd_pcm_info_t *info;
-
-        snd_pcm_info_malloc (&info);
-        snd_pcm_info (sink->handle, info);
-        g_value_set_string (value, snd_pcm_info_get_name (info));
-        snd_pcm_info_free (info);
-      } else {
-        g_value_set_string (value, NULL);
-      }
+      g_value_take_string (value,
+          gst_alsa_find_device_name (GST_OBJECT_CAST (sink),
+              sink->device, sink->handle, SND_PCM_STREAM_PLAYBACK));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -795,7 +780,7 @@ gst_alsasink_write (GstAudioSink * asink, gpointer data, guint length)
   cptr = length / alsa->bytes_per_sample;
   ptr = data;
 
-  GST_ALSA_LOCK (asink);
+  GST_ALSA_SINK_LOCK (asink);
   while (cptr > 0) {
     err = snd_pcm_writei (alsa->handle, ptr, cptr);
 
@@ -813,13 +798,13 @@ gst_alsasink_write (GstAudioSink * asink, gpointer data, guint length)
     ptr += err * alsa->channels;
     cptr -= err;
   }
-  GST_ALSA_UNLOCK (asink);
+  GST_ALSA_SINK_UNLOCK (asink);
 
   return length - cptr;
 
 write_error:
   {
-    GST_ALSA_UNLOCK (asink);
+    GST_ALSA_SINK_UNLOCK (asink);
     return length;              /* skip one period */
   }
 }
@@ -845,29 +830,29 @@ gst_alsasink_reset (GstAudioSink * asink)
 
   alsa = GST_ALSA_SINK (asink);
 
-  GST_ALSA_LOCK (asink);
+  GST_ALSA_SINK_LOCK (asink);
   GST_DEBUG_OBJECT (alsa, "drop");
   CHECK (snd_pcm_drop (alsa->handle), drop_error);
   GST_DEBUG_OBJECT (alsa, "prepare");
   CHECK (snd_pcm_prepare (alsa->handle), prepare_error);
   GST_DEBUG_OBJECT (alsa, "reset done");
-  GST_ALSA_UNLOCK (asink);
+  GST_ALSA_SINK_UNLOCK (asink);
 
   return;
 
   /* ERRORS */
 drop_error:
   {
-    GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS,
-        ("alsa-reset: pcm drop error: %s", snd_strerror (err)), (NULL));
-    GST_ALSA_UNLOCK (asink);
+    GST_ERROR_OBJECT (alsa, "alsa-reset: pcm drop error: %s",
+        snd_strerror (err));
+    GST_ALSA_SINK_UNLOCK (asink);
     return;
   }
 prepare_error:
   {
-    GST_ELEMENT_ERROR (alsa, RESOURCE, SETTINGS,
-        ("alsa-reset: pcm prepare error: %s", snd_strerror (err)), (NULL));
-    GST_ALSA_UNLOCK (asink);
+    GST_ERROR_OBJECT (alsa, "alsa-reset: pcm prepare error: %s",
+        snd_strerror (err));
+    GST_ALSA_SINK_UNLOCK (asink);
     return;
   }
 }
