@@ -85,6 +85,10 @@ static GstStateChangeReturn gst_base_audio_sink_async_play (GstBaseSink *
     basesink);
 static GstStateChangeReturn gst_base_audio_sink_change_state (GstElement *
     element, GstStateChange transition);
+static gboolean gst_base_audio_sink_activate_pull (GstBaseSink * basesink,
+    gboolean active);
+static gboolean gst_base_audio_sink_query (GstElement * element, GstQuery *
+    query);
 
 static GstClock *gst_base_audio_sink_provide_clock (GstElement * elem);
 static GstClockTime gst_base_audio_sink_get_time (GstClock * clock,
@@ -102,6 +106,7 @@ static void gst_base_audio_sink_get_times (GstBaseSink * bsink,
     GstBuffer * buffer, GstClockTime * start, GstClockTime * end);
 static gboolean gst_base_audio_sink_setcaps (GstBaseSink * bsink,
     GstCaps * caps);
+static void gst_base_audio_sink_fixate (GstBaseSink * bsink, GstCaps * caps);
 
 /* static guint gst_base_audio_sink_signals[LAST_SIGNAL] = { 0 }; */
 
@@ -146,6 +151,7 @@ gst_base_audio_sink_class_init (GstBaseAudioSinkClass * klass)
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_change_state);
   gstelement_class->provide_clock =
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_provide_clock);
+  gstelement_class->query = GST_DEBUG_FUNCPTR (gst_base_audio_sink_query);
 
   gstbasesink_class->event = GST_DEBUG_FUNCPTR (gst_base_audio_sink_event);
   gstbasesink_class->preroll = GST_DEBUG_FUNCPTR (gst_base_audio_sink_preroll);
@@ -153,8 +159,11 @@ gst_base_audio_sink_class_init (GstBaseAudioSinkClass * klass)
   gstbasesink_class->get_times =
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_get_times);
   gstbasesink_class->set_caps = GST_DEBUG_FUNCPTR (gst_base_audio_sink_setcaps);
+  gstbasesink_class->fixate = GST_DEBUG_FUNCPTR (gst_base_audio_sink_fixate);
   gstbasesink_class->async_play =
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_async_play);
+  gstbasesink_class->activate_pull =
+      GST_DEBUG_FUNCPTR (gst_base_audio_sink_activate_pull);
 }
 
 static void
@@ -165,8 +174,15 @@ gst_base_audio_sink_init (GstBaseAudioSink * baseaudiosink,
   baseaudiosink->latency_time = DEFAULT_LATENCY_TIME;
   baseaudiosink->provide_clock = DEFAULT_PROVIDE_CLOCK;
 
-  baseaudiosink->provided_clock = gst_audio_clock_new ("clock",
+  baseaudiosink->provided_clock = gst_audio_clock_new ("GstAudioSinkClock",
       (GstAudioClockGetTimeFunc) gst_base_audio_sink_get_time, baseaudiosink);
+
+  GST_BASE_SINK (baseaudiosink)->can_activate_push = TRUE;
+#if 0
+  /* disabling until we fix state changes so that both READY_TO_PAUSED and
+     PAUSED_TO_PLAYING return SUCCESS */
+  GST_BASE_SINK (baseaudiosink)->can_activate_pull = TRUE;
+#endif
 }
 
 static void
@@ -226,6 +242,73 @@ clock_disabled:
   }
 }
 
+static gboolean
+gst_base_audio_sink_query (GstElement * element, GstQuery * query)
+{
+  gboolean res = FALSE;
+
+  GstBaseAudioSink *basesink = GST_BASE_AUDIO_SINK (element);
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_LATENCY:
+    {
+      gboolean live, us_live;
+      GstClockTime min_l, max_l;
+
+      GST_DEBUG_OBJECT (basesink, "latency query");
+
+      if (!basesink->ringbuffer || !basesink->ringbuffer->spec.rate) {
+        GST_DEBUG_OBJECT (basesink,
+            "we are not yet negotiated, can't report latency yet");
+        res = FALSE;
+        goto done;
+      }
+
+      /* ask parent first, it will do an upstream query for us. */
+      if ((res =
+              gst_base_sink_query_latency (GST_BASE_SINK_CAST (basesink), &live,
+                  &us_live, &min_l, &max_l))) {
+        GstClockTime min_latency, max_latency;
+
+        /* we and upstream are both live, adjust the min_latency */
+        if (live && us_live) {
+          GstRingBufferSpec *spec;
+
+          spec = &basesink->ringbuffer->spec;
+
+          min_latency =
+              gst_util_uint64_scale_int (spec->segtotal * spec->segsize,
+              GST_SECOND, spec->rate * spec->bytes_per_sample);
+          /* we cannot go lower than the buffer size */
+          min_latency = MAX (min_latency, min_l);
+          /* the max latency is the max of the peer, we can delay an infinite
+           * amount of time. */
+          max_latency = max_l;
+
+          GST_DEBUG_OBJECT (basesink,
+              "peer min %" GST_TIME_FORMAT ", our min latency: %"
+              GST_TIME_FORMAT, GST_TIME_ARGS (min_l),
+              GST_TIME_ARGS (min_latency));
+        } else {
+          GST_DEBUG_OBJECT (basesink,
+              "peer or we are not live, don't care about latency");
+          min_latency = 0;
+          max_latency = -1;
+        }
+        gst_query_set_latency (query, live, min_latency, max_latency);
+      }
+      break;
+    }
+    default:
+      res = GST_ELEMENT_CLASS (parent_class)->query (element, query);
+      break;
+  }
+
+done:
+  return res;
+}
+
+
 static GstClockTime
 gst_base_audio_sink_get_time (GstClock * clock, GstBaseAudioSink * sink)
 {
@@ -368,6 +451,32 @@ acquire_error:
   }
 }
 
+static void
+gst_base_audio_sink_fixate (GstBaseSink * bsink, GstCaps * caps)
+{
+  GstStructure *s;
+  gint width, depth;
+
+  s = gst_caps_get_structure (caps, 0);
+
+  /* fields for all formats */
+  gst_structure_fixate_field_nearest_int (s, "rate", 44100);
+  gst_structure_fixate_field_nearest_int (s, "channels", 2);
+  gst_structure_fixate_field_nearest_int (s, "width", 16);
+
+  /* fields for int */
+  if (gst_structure_has_field (s, "depth")) {
+    gst_structure_get_int (s, "width", &width);
+    /* round width to nearest multiple of 8 for the depth */
+    depth = GST_ROUND_UP_8 (width);
+    gst_structure_fixate_field_nearest_int (s, "depth", depth);
+  }
+  if (gst_structure_has_field (s, "signed"))
+    gst_structure_fixate_field_boolean (s, "signed", TRUE);
+  if (gst_structure_has_field (s, "endianness"))
+    gst_structure_fixate_field_nearest_int (s, "endianness", G_BYTE_ORDER);
+}
+
 static void
 gst_base_audio_sink_get_times (GstBaseSink * bsink, GstBuffer * buffer,
     GstClockTime * start, GstClockTime * end)
@@ -449,7 +558,7 @@ gst_base_audio_sink_event (GstBaseSink * bsink, GstEvent * event)
       gst_event_parse_new_segment_full (event, NULL, &rate, NULL, NULL,
           NULL, NULL, NULL);
 
-      GST_DEBUG_OBJECT (sink, "new rate of %f", rate);
+      GST_DEBUG_OBJECT (sink, "new segment rate of %f", rate);
       break;
     }
     default:
@@ -515,7 +624,7 @@ gst_base_audio_sink_get_offset (GstBaseAudioSink * sink)
 static GstFlowReturn
 gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
 {
-  guint64 in_offset, clock_offset;
+  guint64 in_offset;
   GstClockTime time, stop, render_start, render_stop, sample_offset;
   GstBaseAudioSink *sink;
   GstRingBuffer *ringbuf;
@@ -528,9 +637,9 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   GstClockTime crate_num;
   GstClockTime crate_denom;
   gint out_samples;
-  GstClockTime cinternal, cexternal;
+  GstClockTime base_time, cinternal, cexternal, latency;
   GstClock *clock;
-  gboolean sync;
+  gboolean sync, slaved;
 
   sink = GST_BASE_AUDIO_SINK (bsink);
 
@@ -628,29 +737,84 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
       "running: start %" GST_TIME_FORMAT " - stop %" GST_TIME_FORMAT,
       GST_TIME_ARGS (render_start), GST_TIME_ARGS (render_stop));
 
-  /* get calibration parameters to compensate for speed and offset differences
-   * when we are slaved */
-  gst_clock_get_calibration (sink->provided_clock, &cinternal, &cexternal,
-      &crate_num, &crate_denom);
+  base_time = gst_element_get_base_time (GST_ELEMENT_CAST (bsink));
 
-  clock_offset =
-      (gst_element_get_base_time (GST_ELEMENT_CAST (bsink)) - cexternal) +
-      cinternal;
+  GST_DEBUG_OBJECT (sink, "base_time %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (base_time));
+
+  /* add base time to sync against the clock */
+  render_start += base_time;
+  render_stop += base_time;
+
+  /* compensate for latency */
+  latency = gst_base_sink_get_latency (bsink);
+  GST_DEBUG_OBJECT (sink,
+      "compensating for latency %" GST_TIME_FORMAT, GST_TIME_ARGS (latency));
+
+  slaved = clock != sink->provided_clock;
+  if (slaved) {
+    /* get calibration parameters to compensate for speed and offset differences
+     * when we are slaved */
+    gst_clock_get_calibration (sink->provided_clock, &cinternal, &cexternal,
+        &crate_num, &crate_denom);
+
+    cinternal += latency;
+
+    GST_DEBUG_OBJECT (sink, "internal %" GST_TIME_FORMAT " external %"
+        GST_TIME_FORMAT " %" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT " = %f",
+        GST_TIME_ARGS (cinternal), GST_TIME_ARGS (cexternal), crate_num,
+        crate_denom, (gdouble) crate_num / crate_denom);
+
+    if (crate_num == 0)
+      crate_denom = crate_num = 1;
+
+    /* bring to our slaved clock time */
+    if (render_start >= cexternal) {
+      render_start =
+          gst_util_uint64_scale (render_start - cexternal, crate_denom,
+          crate_num);
+      render_start += cinternal;
+    } else {
+      render_start = gst_util_uint64_scale (cexternal - render_start,
+          crate_denom, crate_num);
+      if (cinternal > render_start)
+        render_start = cinternal - render_start;
+      else
+        render_start = 0;
+    }
+
+    if (render_stop >= cexternal) {
+      render_stop =
+          gst_util_uint64_scale (render_stop - cexternal, crate_denom,
+          crate_num);
+      render_stop += cinternal;
+    } else {
+      render_stop = gst_util_uint64_scale (cexternal - render_stop,
+          crate_denom, crate_num);
+      if (cinternal > render_stop)
+        render_stop = cinternal - render_stop;
+      else
+        render_stop = 0;
+    }
+
+    GST_DEBUG_OBJECT (sink,
+        "after slaving: start %" GST_TIME_FORMAT " - stop %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (render_start), GST_TIME_ARGS (render_stop));
+  } else {
+    render_start += latency;
+    render_stop += latency;
+    GST_DEBUG_OBJECT (sink,
+        "after latency: start %" GST_TIME_FORMAT " - stop %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (render_start), GST_TIME_ARGS (render_stop));
+  }
 
-  GST_DEBUG_OBJECT (sink, "clock offset %" GST_TIME_FORMAT " %" G_GUINT64_FORMAT
-      "/%" G_GUINT64_FORMAT, GST_TIME_ARGS (clock_offset), crate_num,
-      crate_denom);
 
   /* and bring the time to the rate corrected offset in the buffer */
-  render_start = gst_util_uint64_scale_int (render_start + clock_offset,
+  render_start = gst_util_uint64_scale_int (render_start,
       ringbuf->spec.rate, GST_SECOND);
-  render_stop = gst_util_uint64_scale_int (render_stop + clock_offset,
+  render_stop = gst_util_uint64_scale_int (render_stop,
       ringbuf->spec.rate, GST_SECOND);
 
-  GST_DEBUG_OBJECT (sink,
-      "render: start %" GST_TIME_FORMAT " - stop %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (render_start), GST_TIME_ARGS (render_stop));
-
   /* always resync after a discont */
   if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT))) {
     GST_DEBUG_OBJECT (sink, "resync after discont");
@@ -701,7 +865,7 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   render_start += align;
 
   /* only align stop if we are not slaved */
-  if (clock != sink->provided_clock) {
+  if (slaved) {
     GST_DEBUG_OBJECT (sink, "no stop time align needed: we are slaved");
     goto no_align;
   }
@@ -810,11 +974,63 @@ gst_base_audio_sink_create_ringbuffer (GstBaseAudioSink * sink)
   return buffer;
 }
 
+static gboolean
+gst_base_audio_sink_activate_pull (GstBaseSink * basesink, gboolean active)
+{
+  gboolean ret;
+  GstBaseAudioSink *sink = GST_BASE_AUDIO_SINK (basesink);
+
+  if (active) {
+    gst_ring_buffer_set_callback (sink->ringbuffer,
+        gst_base_audio_sink_callback, sink);
+    ret = gst_ring_buffer_start (sink->ringbuffer);
+  } else {
+    gst_ring_buffer_set_callback (sink->ringbuffer, NULL, NULL);
+    /* stop thread */
+    ret = gst_ring_buffer_release (sink->ringbuffer);
+  }
+
+  return ret;
+}
+
 static void
 gst_base_audio_sink_callback (GstRingBuffer * rbuf, guint8 * data, guint len,
     gpointer user_data)
 {
-  /* GstBaseAudioSink *sink = GST_BASE_AUDIO_SINK (data); */
+  GstBaseSink *basesink;
+  GstBaseAudioSink *sink;
+  GstBuffer *buf;
+  GstFlowReturn ret;
+
+  basesink = GST_BASE_SINK (user_data);
+  sink = GST_BASE_AUDIO_SINK (user_data);
+
+  /* would be nice to arrange for pad_alloc_buffer to return data -- as it is we
+     will copy twice, once into data, once into DMA */
+  GST_LOG_OBJECT (basesink, "pulling %d bytes offset %" G_GUINT64_FORMAT
+      " to fill audio buffer", len, basesink->offset);
+  ret = gst_pad_pull_range (basesink->sinkpad, basesink->offset, len, &buf);
+  if (ret != GST_FLOW_OK)
+    goto error;
+
+  if (len != GST_BUFFER_SIZE (buf)) {
+    GST_INFO_OBJECT (basesink, "short read pulling from sink pad: %d<%d",
+        len, GST_BUFFER_SIZE (buf));
+    len = MIN (GST_BUFFER_SIZE (buf), len);
+  }
+
+  basesink->offset += len;
+
+  memcpy (data, GST_BUFFER_DATA (buf), len);
+
+  return;
+
+error:
+  {
+    GST_WARNING_OBJECT (basesink, "Got flow error but can't return it: %d",
+        ret);
+    return;
+  }
 }
 
 /* should be called with the LOCK */
@@ -822,7 +1038,7 @@ static GstStateChangeReturn
 gst_base_audio_sink_async_play (GstBaseSink * basesink)
 {
   GstClock *clock;
-  GstClockTime time, base;
+  GstClockTime itime, etime;
   GstBaseAudioSink *sink;
 
   sink = GST_BASE_AUDIO_SINK (basesink);
@@ -840,23 +1056,25 @@ gst_base_audio_sink_async_play (GstBaseSink * basesink)
   if (clock != sink->provided_clock) {
     GstClockTime rate_num, rate_denom;
 
-    base = GST_ELEMENT_CAST (sink)->base_time;
-    time = gst_clock_get_internal_time (sink->provided_clock);
+    etime = gst_clock_get_time (clock);
+    itime = gst_clock_get_internal_time (sink->provided_clock);
 
     GST_DEBUG_OBJECT (sink,
-        "time: %" GST_TIME_FORMAT " base: %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (time), GST_TIME_ARGS (base));
+        "internal time: %" GST_TIME_FORMAT " external time: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (itime), GST_TIME_ARGS (etime));
 
     /* FIXME, this is not yet accurate enough for smooth playback */
     gst_clock_get_calibration (sink->provided_clock, NULL, NULL, &rate_num,
         &rate_denom);
     /* Does not work yet. */
-    gst_clock_set_calibration (sink->provided_clock, time, base,
+    gst_clock_set_calibration (sink->provided_clock, itime, etime,
         rate_num, rate_denom);
 
     gst_clock_set_master (sink->provided_clock, clock);
-  }
 
+    /* start ringbuffer so we can start slaving right away */
+    gst_ring_buffer_start (sink->ringbuffer);
+  }
 no_clock:
   return GST_STATE_CHANGE_SUCCESS;
 }
@@ -884,8 +1102,6 @@ gst_base_audio_sink_change_state (GstElement * element,
     case GST_STATE_CHANGE_NULL_TO_READY:
       if (sink->ringbuffer == NULL) {
         sink->ringbuffer = gst_base_audio_sink_create_ringbuffer (sink);
-        gst_ring_buffer_set_callback (sink->ringbuffer,
-            gst_base_audio_sink_callback, sink);
       }
       if (!gst_ring_buffer_open_device (sink->ringbuffer))
         goto open_failed;
@@ -925,7 +1141,6 @@ gst_base_audio_sink_change_state (GstElement * element,
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       gst_ring_buffer_release (sink->ringbuffer);
-      gst_pad_set_caps (GST_BASE_SINK_PAD (sink), NULL);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       gst_ring_buffer_close_device (sink->ringbuffer);
