@@ -30,19 +30,19 @@ G_BEGIN_DECLS
 #define GST_BASE_RTP_DEPAYLOAD(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_BASE_RTP_DEPAYLOAD,GstBaseRTPDepayload))
 #define GST_BASE_RTP_DEPAYLOAD_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_BASE_RTP_DEPAYLOAD,GstBaseRTPDepayload))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_BASE_RTP_DEPAYLOAD,GstBaseRTPDepayloadClass))
 #define GST_BASE_RTP_DEPAYLOAD_GET_CLASS(obj) \
         (G_TYPE_INSTANCE_GET_CLASS ((obj),GST_TYPE_BASE_RTP_DEPAYLOAD,GstBaseRTPDepayloadClass))
 #define GST_IS_BASE_RTP_DEPAYLOAD(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_BASE_RTP_DEPAYLOAD))
-#define GST_IS_BASE_RTP_DEPAYLOAD_CLASS(obj) \
+#define GST_IS_BASE_RTP_DEPAYLOAD_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_BASE_RTP_DEPAYLOAD))
 
 #define GST_BASE_RTP_DEPAYLOAD_SINKPAD(depayload) (GST_BASE_RTP_DEPAYLOAD (depayload)->sinkpad)
 #define GST_BASE_RTP_DEPAYLOAD_SRCPAD(depayload)  (GST_BASE_RTP_DEPAYLOAD (depayload)->srcpad)
 
 /* in milliseconds */
-#define RTP_QUEUE_DELAY 100;
+#define RTP_QUEUE_DELAY 0;
 
 #define QUEUE_LOCK_INIT(base)   (g_static_rec_mutex_init(&base->queuelock))
 #define QUEUE_LOCK_FREE(base)   (g_static_rec_mutex_free(&base->queuelock))
@@ -98,7 +98,9 @@ struct _GstBaseRTPDepayloadClass
   GstFlowReturn (*add_to_queue) (GstBaseRTPDepayload *filter, GstBuffer *in);
 
   /* pure virtual function, child must use this to process incoming
-   * rtp packets
+   * rtp packets. If the child returns a buffer, the timestamp of @in will be
+   * applied to the result buffer and the buffer will be pushed. If this
+   * function returns %NULL, nothing is pushed.
    */
   GstBuffer * (*process) (GstBaseRTPDepayload *base, GstBuffer *in);
 
@@ -113,6 +115,10 @@ struct _GstBaseRTPDepayloadClass
 
 GType gst_base_rtp_depayload_get_type (void);
 
+GstFlowReturn   gst_base_rtp_depayload_push              (GstBaseRTPDepayload *filter, GstBuffer *out_buf);
+GstFlowReturn   gst_base_rtp_depayload_push_ts           (GstBaseRTPDepayload *filter,
+		                                          guint32 timestamp, GstBuffer *out_buf);
+
 G_END_DECLS
 
 #endif /* __GST_BASE_RTP_DEPAYLOAD_H__ */
