@@ -41,6 +41,9 @@
 #include <stdlib.h>
 #include <string.h>
 
+/*
+ * see http://xiph.org/ogg/vorbis/doc/v-comment.html
+ */
 static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_TITLE, "TITLE"},
   {GST_TAG_VERSION, "VERSION"},
@@ -53,6 +56,7 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_PERFORMER, "PERFORMER"},
   {GST_TAG_COPYRIGHT, "COPYRIGHT"},
   {GST_TAG_LICENSE, "LICENSE"},
+  {GST_TAG_LOCATION, "LOCATION"},
   {GST_TAG_ORGANIZATION, "ORGANIZATION"},
   {GST_TAG_DESCRIPTION, "DESCRIPTION"},
   {GST_TAG_GENRE, "GENRE"},
@@ -64,6 +68,7 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_TRACK_PEAK, "REPLAYGAIN_TRACK_PEAK"},
   {GST_TAG_ALBUM_GAIN, "REPLAYGAIN_ALBUM_GAIN"},
   {GST_TAG_ALBUM_PEAK, "REPLAYGAIN_ALBUM_PEAK"},
+  {GST_TAG_REFERENCE_LEVEL, "REPLAYGAIN_REFERENCE_LOUDNESS"},
   {GST_TAG_MUSICBRAINZ_TRACKID, "MUSICBRAINZ_TRACKID"},
   {GST_TAG_MUSICBRAINZ_ARTISTID, "MUSICBRAINZ_ARTISTID"},
   {GST_TAG_MUSICBRAINZ_ALBUMID, "MUSICBRAINZ_ALBUMID"},
@@ -73,6 +78,10 @@ static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_LANGUAGE_CODE, "LANGUAGE"},
   {GST_TAG_CDDA_MUSICBRAINZ_DISCID, "MUSICBRAINZ_DISCID"},
   {GST_TAG_CDDA_CDDB_DISCID, "DISCID"},
+  /* some incidence that this makes sense:
+   * http://mail.kde.org/pipermail/amarok/2006-May/000090.html
+   */
+  {GST_TAG_BEATS_PER_MINUTE, "BPM"},
   {NULL, NULL}
 };
 
@@ -80,7 +89,7 @@ static const GstTagEntryMatch tag_matches[] = {
  * gst_tag_from_vorbis_tag:
  * @vorbis_tag: vorbiscomment tag to convert to GStreamer tag
  *
- * Looks up the GStreamer tag for a vorbiscommment tag.
+ * Looks up the GStreamer tag for a vorbiscomment tag.
  *
  * Returns: The corresponding GStreamer tag or NULL if none exists.
  */
@@ -109,9 +118,9 @@ gst_tag_from_vorbis_tag (const gchar * vorbis_tag)
  * gst_tag_to_vorbis_tag:
  * @gst_tag: GStreamer tag to convert to vorbiscomment tag
  *
- * Looks up the vorbiscommment tag for a GStreamer tag.
+ * Looks up the vorbiscomment tag for a GStreamer tag.
  *
- * Returns: The corresponding vorbiscommment tag or NULL if none exists.
+ * Returns: The corresponding vorbiscomment tag or NULL if none exists.
  */
 G_CONST_RETURN gchar *
 gst_tag_to_vorbis_tag (const gchar * gst_tag)
@@ -245,6 +254,12 @@ gst_vorbis_tag_add (GstTagList * list, const gchar * tag, const gchar * value)
           }
         }
 
+        if (y != 0 && m == 0 && d == 0) {
+          /* accept dates of the form 2007-00-00 as 2007-01-01 */
+          m = 1;
+          d = 1;
+        }
+
         /* date might be followed by a time */
         if ((*check == '\0' || g_ascii_isspace (*check)) && y != 0 &&
             g_date_valid_dmy (d, m, y)) {
@@ -508,12 +523,12 @@ gst_tag_list_to_vorbiscomment_buffer (const GstTagList * list,
     memcpy (data, id_data, id_data_length);
     data += id_data_length;
   }
-  *((guint32 *) data) = GUINT32_TO_LE (vendor_len);
+  GST_WRITE_UINT32_LE (data, vendor_len);
   data += 4;
   memcpy (data, vendor_string, vendor_len);
   data += vendor_len;
   l = my_data.entries = g_list_reverse (my_data.entries);
-  *((guint32 *) data) = GUINT32_TO_LE (my_data.count);
+  GST_WRITE_UINT32_LE (data, my_data.count);
   data += 4;
   for (i = 0; i < my_data.count; i++) {
     guint size;
@@ -523,7 +538,7 @@ gst_tag_list_to_vorbiscomment_buffer (const GstTagList * list,
     cur = l->data;
     l = g_list_next (l);
     size = strlen (cur);
-    *((guint32 *) data) = GUINT32_TO_LE (size);
+    GST_WRITE_UINT32_LE (data, size);
     data += 4;
     memcpy (data, cur, size);
     data += size;
