@@ -19,7 +19,17 @@
 
 /**
  * SECTION:element-decodebin2
+ * @short_description: Next-generation automatic decoding bin
  *
+ * #GstBin that auto-magically constructs a decoding pipeline using available
+ * decoders and demuxers via auto-plugging.
+ *
+ * At this stage, decodebin2 is considered UNSTABLE. The API provided in the
+ * signals is expected to change in the near future. 
+ *
+ * To try out decodebin2, you can set the USE_DECODEBIN2 environment 
+ * variable (USE_DECODEBIN2=1 for example). This will cause playbin to use
+ * decodebin2 instead of the older decodebin for its internal auto-plugging.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +38,7 @@
 
 #include <string.h>
 #include <gst/gst.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "gstplay-marshal.h"
 
@@ -50,14 +61,21 @@ GST_DEBUG_CATEGORY_STATIC (gst_decode_bin_debug);
 typedef struct _GstDecodeGroup GstDecodeGroup;
 typedef struct _GstDecodePad GstDecodePad;
 typedef struct _GstDecodeBin GstDecodeBin;
+typedef struct _GstDecodeBin GstDecodeBin2;
 typedef struct _GstDecodeBinClass GstDecodeBinClass;
 
 #define GST_TYPE_DECODE_BIN             (gst_decode_bin_get_type())
+#define GST_DECODE_BIN_CAST(obj)        ((GstDecodeBin*)(obj))
 #define GST_DECODE_BIN(obj)             (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_DECODE_BIN,GstDecodeBin))
 #define GST_DECODE_BIN_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DECODE_BIN,GstDecodeBinClass))
 #define GST_IS_DECODE_BIN(obj)          (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_DECODE_BIN))
 #define GST_IS_DECODE_BIN_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_DECODE_BIN))
 
+/**
+ *  GstDecodeBin2:
+ *
+ *  The opaque #DecodeBin2 data structure
+ */
 struct _GstDecodeBin
 {
   GstBin bin;                   /* we extend GstBin */
@@ -69,6 +87,8 @@ struct _GstDecodeBin
   GstDecodeGroup *activegroup;  /* group currently active */
   GList *groups;                /* List of non-active GstDecodeGroups, sorted in
                                  * order of creation. */
+  GList *oldgroups;             /* List of no-longer-used GstDecodeGroups. 
+                                 * Should be freed in dispose */
   gint nbpads;                  /* unique identifier for source pads */
   GstCaps *caps;                /* caps on which to stop decoding */
 
@@ -120,7 +140,7 @@ GST_ELEMENT_DETAILS ("Decoder Bin",
     "Edward Hervey <edward@fluendo.com>");
 
 
-static void add_fakesink (GstDecodeBin * decode_bin);
+static gboolean add_fakesink (GstDecodeBin * decode_bin);
 static void remove_fakesink (GstDecodeBin * decode_bin);
 
 static void type_found (GstElement * typefind, guint probability,
@@ -145,7 +165,7 @@ static GstStateChangeReturn gst_decode_bin_change_state (GstElement * element,
     GST_LOG_OBJECT (dbin,						\
 		    "locking from thread %p",				\
 		    g_thread_self ());					\
-    g_mutex_lock (dbin->lock);						\
+    g_mutex_lock (GST_DECODE_BIN_CAST(dbin)->lock);			\
     GST_LOG_OBJECT (dbin,						\
 		    "locked from thread %p",				\
 		    g_thread_self ());					\
@@ -155,7 +175,7 @@ static GstStateChangeReturn gst_decode_bin_change_state (GstElement * element,
     GST_LOG_OBJECT (dbin,						\
 		    "unlocking from thread %p",				\
 		    g_thread_self ());					\
-    g_mutex_unlock (dbin->lock);					\
+    g_mutex_unlock (GST_DECODE_BIN_CAST(dbin)->lock);			\
 } G_STMT_END
 
 /* GstDecodeGroup
@@ -174,6 +194,10 @@ struct _GstDecodeGroup
   gboolean blocked;             /* TRUE if all endpads are blocked */
   gboolean complete;            /* TRUE if we are not expecting anymore streams 
                                  * on this group */
+  gulong overrunsig;
+  gulong underrunsig;
+  guint nbdynamic;              /* number of dynamic pads in the group. */
+
   GList *endpads;               /* List of GstDecodePad of source pads to be exposed */
   GList *ghosts;                /* List of GstGhostPad for the endpads */
 };
@@ -302,27 +326,85 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
   gobject_klass->set_property = GST_DEBUG_FUNCPTR (gst_decode_bin_set_property);
   gobject_klass->get_property = GST_DEBUG_FUNCPTR (gst_decode_bin_get_property);
 
+  /**
+   * GstDecodeBin2::new-decoded-pad:
+   * @pad: the newly created pad
+   * @islast: #TRUE if this is the last pad to be added. Deprecated.
+   *
+   * This signal gets emitted as soon as a new pad of the same type as one of
+   * the valid 'raw' types is added.
+   */
+
   gst_decode_bin_signals[SIGNAL_NEW_DECODED_PAD] =
       g_signal_new ("new-decoded-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstDecodeBinClass, new_decoded_pad), NULL, NULL,
       gst_play_marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2, GST_TYPE_PAD,
       G_TYPE_BOOLEAN);
+
+  /**
+   * GstDecodeBin2::removed-decoded-pad:
+   * @pad: the pad that was removed
+   *
+   * This signal is emitted when a 'final' caps pad has been removed.
+   */
+
   gst_decode_bin_signals[SIGNAL_REMOVED_DECODED_PAD] =
       g_signal_new ("removed-decoded-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstDecodeBinClass, removed_decoded_pad), NULL, NULL,
       gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_PAD);
+
+  /**
+   * GstDecodeBin2::unknown-type:
+   * @pad: the new pad containing caps that cannot be resolved to a 'final' stream type.
+   * @caps: the #GstCaps of the pad that cannot be resolved.
+   *
+   * This signal is emitted when a pad for which there is no further possible
+   * decoding is added to the decodebin.
+   */
+
   gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE] =
       g_signal_new ("unknown-type", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, unknown_type),
       NULL, NULL, gst_marshal_VOID__OBJECT_OBJECT, G_TYPE_NONE, 2,
       GST_TYPE_PAD, GST_TYPE_CAPS);
+
+  /**
+   * GstDecodeBin2::autoplug-continue:
+   * @caps: The #GstCaps found.
+   *
+   * This signal is emitted whenever decodebin2 finds a new stream. It is
+   * emitted before looking for any elements that can handle that stream.
+   *
+   * Returns: #TRUE if you wish decodebin2 to look for elements that can
+   * handle the given @caps. If #FALSE, those caps will be considered as
+   * final and the pad will be exposed as such (see 'new-decoded-pad'
+   * signal).
+   */
+
   gst_decode_bin_signals[SIGNAL_AUTOPLUG_CONTINUE] =
       g_signal_new ("autoplug-continue", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, autoplug_continue),
       _gst_boolean_accumulator, NULL, gst_play_marshal_BOOLEAN__OBJECT,
       G_TYPE_BOOLEAN, 1, GST_TYPE_CAPS);
+
+  /**
+   * GstDecodeBin2::autoplug-sort:
+   * @caps: The #GstCaps.
+   * @factories: A #GList of possible #GstElementFactory to use.
+   *
+   * This signal is emitted once decodebin2 has found all the possible
+   * #GstElementFactory that can be used to handle the given @caps.
+   *
+   * UNSTABLE API. Will change soon.
+   *
+   * Returns: #TRUE if you wish decodebin2 to start trying to decode
+   * the given @caps with the list of factories. #FALSE if you do not want
+   * these #GstCaps, if so the pad will be exposed as unknown (see
+   * 'unknown-type' signal).
+   */
+
   gst_decode_bin_signals[SIGNAL_AUTOPLUG_SORT] =
       g_signal_new ("autoplug-sort", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, autoplug_sort),
@@ -469,6 +551,7 @@ static void
 gst_decode_bin_dispose (GObject * object)
 {
   GstDecodeBin *decode_bin;
+  GList *tmp;
 
   decode_bin = GST_DECODE_BIN (object);
 
@@ -476,7 +559,28 @@ gst_decode_bin_dispose (GObject * object)
     gst_plugin_feature_list_free (decode_bin->factories);
   decode_bin->factories = NULL;
 
-  /* FILLME */
+  if (decode_bin->activegroup) {
+    gst_decode_group_free (decode_bin->activegroup);
+    decode_bin->activegroup = NULL;
+  }
+
+  /* remove groups */
+  for (tmp = decode_bin->groups; tmp; tmp = g_list_next (tmp)) {
+    GstDecodeGroup *group = (GstDecodeGroup *) tmp->data;
+
+    gst_decode_group_free (group);
+  }
+  g_list_free (decode_bin->groups);
+  decode_bin->groups = NULL;
+
+  for (tmp = decode_bin->oldgroups; tmp; tmp = g_list_next (tmp)) {
+    GstDecodeGroup *group = (GstDecodeGroup *) tmp->data;
+
+    gst_decode_group_free (group);
+  }
+  g_list_free (decode_bin->oldgroups);
+  decode_bin->oldgroups = NULL;
+
   if (decode_bin->caps)
     gst_caps_unref (decode_bin->caps);
   decode_bin->caps = NULL;
@@ -488,8 +592,14 @@ gst_decode_bin_dispose (GObject * object)
 static void
 gst_decode_bin_finalize (GObject * object)
 {
+  GstDecodeBin *decode_bin;
 
-  /* FILLME */
+  decode_bin = GST_DECODE_BIN (object);
+
+  if (decode_bin->lock) {
+    g_mutex_free (decode_bin->lock);
+    decode_bin->lock = NULL;
+  }
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -691,9 +801,13 @@ unknown_type:
     GST_LOG_OBJECT (pad, "Unknown type, firing signal");
     g_signal_emit (G_OBJECT (dbin),
         gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE], 0, pad, caps);
+
     /* Check if there are no pending groups, if so, remove fakesink */
     if (dbin->groups == NULL)
       remove_fakesink (dbin);
+
+    gst_element_post_message (GST_ELEMENT_CAST (dbin),
+        gst_missing_decoder_message_new (GST_ELEMENT_CAST (dbin), caps));
     return;
   }
 
@@ -707,7 +821,7 @@ any_caps:
 }
 
 
-/** connect_pad:
+/* connect_pad:
  *
  * Try to connect the given pad to an element created from one of the factories,
  * and recursively.
@@ -893,6 +1007,12 @@ connect_element (GstDecodeBin * dbin, GstElement * element,
   /* 2. if there are more potential pads, connect to relevent signals */
   if (dynamic) {
     if (group) {
+      GST_LOG ("Adding signals to element %s in group %p",
+          GST_ELEMENT_NAME (element), group);
+      GROUP_MUTEX_LOCK (group);
+      group->nbdynamic++;
+      GST_LOG ("Group %p has now %d dynamic elements", group, group->nbdynamic);
+      GROUP_MUTEX_UNLOCK (group);
       g_signal_connect (G_OBJECT (element), "pad-added",
           G_CALLBACK (pad_added_group_cb), group);
       g_signal_connect (G_OBJECT (element), "pad-removed",
@@ -998,9 +1118,29 @@ type_found (GstElement * typefind, guint probability,
 static void
 pad_added_group_cb (GstElement * element, GstPad * pad, GstDecodeGroup * group)
 {
+  GstCaps *caps;
+  gboolean expose = FALSE;
+
   GST_LOG_OBJECT (pad, "pad added, group:%p", group);
 
-  /* FIXME : FILLME */
+  caps = gst_pad_get_caps (pad);
+  analyze_new_pad (group->dbin, element, pad, caps, group);
+  if (caps)
+    gst_caps_unref (caps);
+
+  GROUP_MUTEX_LOCK (group);
+  group->nbdynamic--;
+  GST_LOG ("Group %p has now %d dynamic objects", group, group->nbdynamic);
+  if (group->nbdynamic == 0)
+    expose = TRUE;
+  GROUP_MUTEX_UNLOCK (group);
+  if (expose) {
+    GST_LOG
+        ("That was the last dynamic object, now attempting to expose the group");
+    DECODE_BIN_LOCK (group->dbin);
+    gst_decode_group_expose (group);
+    DECODE_BIN_UNLOCK (group->dbin);
+  }
 }
 
 static void
@@ -1213,7 +1353,6 @@ multi_queue_underrun_cb (GstElement * queue, GstDecodeGroup * group)
     GST_DEBUG_OBJECT (dbin, "Switching to new group");
     /* unexpose current active */
     gst_decode_group_hide (group);
-    gst_decode_group_free (group);
 
     /* expose first group of groups */
     gst_decode_group_expose ((GstDecodeGroup *) dbin->groups->data);
@@ -1240,8 +1379,8 @@ gst_decode_group_new (GstDecodeBin * dbin)
   }
 
   g_object_set (G_OBJECT (mq),
-      "max-size-bytes", 1 * 1024 * 1024,
-      "max-size-time", G_GINT64_CONSTANT (0), "max-size-buffers", 0, NULL);
+      "max-size-bytes", 2 * 1024 * 1024,
+      "max-size-time", 5 * GST_SECOND, "max-size-buffers", 0, NULL);
 
   group = g_new0 (GstDecodeGroup, 1);
   group->lock = g_mutex_new ();
@@ -1253,9 +1392,9 @@ gst_decode_group_new (GstDecodeBin * dbin)
   group->complete = FALSE;
   group->endpads = NULL;
 
-  g_signal_connect (G_OBJECT (mq), "overrun",
+  group->overrunsig = g_signal_connect (G_OBJECT (mq), "overrun",
       G_CALLBACK (multi_queue_overrun_cb), group);
-  g_signal_connect (G_OBJECT (mq), "underrun",
+  group->underrunsig = g_signal_connect (G_OBJECT (mq), "underrun",
       G_CALLBACK (multi_queue_underrun_cb), group);
 
   gst_bin_add (GST_BIN (dbin), group->multiqueue);
@@ -1282,6 +1421,8 @@ get_current_group (GstDecodeBin * dbin)
   for (tmp = dbin->groups; tmp; tmp = g_list_next (tmp)) {
     GstDecodeGroup *this = (GstDecodeGroup *) tmp->data;
 
+    GST_LOG_OBJECT (dbin, "group %p, complete:%d", this, this->complete);
+
     if (!this->complete) {
       group = this;
       break;
@@ -1301,7 +1442,9 @@ group_demuxer_event_probe (GstPad * pad, GstEvent * event,
   if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
     GST_DEBUG_OBJECT (group->dbin,
         "Got EOS on group input pads, exposing group if it wasn't before");
+    DECODE_BIN_LOCK (group->dbin);
     gst_decode_group_expose (group);
+    DECODE_BIN_UNLOCK (group->dbin);
   }
   return TRUE;
 }
@@ -1385,6 +1528,8 @@ gst_decode_group_control_source_pad (GstDecodeGroup * group, GstPad * pad)
  * Call this when one of the pads blocked status has changed.
  * If the group is complete and blocked, the group will be marked as blocked
  * and will ghost/expose all pads on decodebin if the group is the current one.
+ *
+ * Call with the group lock taken ! MT safe
  */
 static void
 gst_decode_group_check_if_blocked (GstDecodeGroup * group)
@@ -1392,10 +1537,11 @@ gst_decode_group_check_if_blocked (GstDecodeGroup * group)
   GList *tmp;
   gboolean blocked = TRUE;
 
-  GST_LOG ("group : %p");
+  GST_LOG ("group : %p , ->complete:%d , ->nbdynamic:%d",
+      group, group->complete, group->nbdynamic);
 
   /* 1. don't do anything if group is not complete */
-  if (!group->complete) {
+  if (!group->complete || group->nbdynamic) {
     GST_DEBUG_OBJECT (group->dbin, "Group isn't complete yet");
     return;
   }
@@ -1455,13 +1601,70 @@ gst_decode_group_check_if_drained (GstDecodeGroup * group)
     GST_DEBUG_OBJECT (dbin, "Switching to new group");
 
     gst_decode_group_hide (group);
-    gst_decode_group_free (group);
 
     gst_decode_group_expose ((GstDecodeGroup *) dbin->groups->data);
   }
   DECODE_BIN_UNLOCK (dbin);
 }
 
+/* sort_end_pads:
+ * GCompareFunc to use with lists of GstPad.
+ * Sorts pads by mime type.
+ * First video (raw, then non-raw), then audio (raw, then non-raw),
+ * then others.
+ *
+ * Return: negative if a<b, 0 if a==b, positive if a>b
+ */
+
+static gint
+sort_end_pads (GstDecodePad * da, GstDecodePad * db)
+{
+  GstPad *a, *b;
+  gint va, vb;
+  GstCaps *capsa, *capsb;
+  GstStructure *sa, *sb;
+  const gchar *namea, *nameb;
+
+  a = da->pad;
+  b = db->pad;
+
+  capsa = gst_pad_get_caps (a);
+  capsb = gst_pad_get_caps (b);
+
+  sa = gst_caps_get_structure ((const GstCaps *) capsa, 0);
+  sb = gst_caps_get_structure ((const GstCaps *) capsb, 0);
+
+  namea = gst_structure_get_name (sa);
+  nameb = gst_structure_get_name (sb);
+
+  if (g_strrstr (namea, "video/x-raw-"))
+    va = 0;
+  else if (g_strrstr (namea, "video/"))
+    va = 1;
+  else if (g_strrstr (namea, "audio/x-raw"))
+    va = 2;
+  else if (g_strrstr (namea, "audio/"))
+    va = 3;
+  else
+    va = 4;
+
+  if (g_strrstr (nameb, "video/x-raw-"))
+    vb = 0;
+  else if (g_strrstr (nameb, "video/"))
+    vb = 1;
+  else if (g_strrstr (nameb, "audio/x-raw"))
+    vb = 2;
+  else if (g_strrstr (nameb, "audio/"))
+    vb = 3;
+  else
+    vb = 4;
+
+  gst_caps_unref (capsa);
+  gst_caps_unref (capsb);
+
+  return va - vb;
+}
+
 /* gst_decode_group_expose:
  *
  * Expose this group's pads.
@@ -1477,25 +1680,31 @@ gst_decode_group_expose (GstDecodeGroup * group)
 
   if (group->dbin->activegroup) {
     GST_DEBUG_OBJECT (group->dbin, "A group is already active and exposed");
-    DECODE_BIN_UNLOCK (group->dbin);
     return TRUE;
   }
 
   if (group->dbin->activegroup == group) {
     GST_WARNING ("Group %p is already exposed", group);
-    DECODE_BIN_UNLOCK (group->dbin);
     return TRUE;
   }
 
   if (!group->dbin->groups
       || (group != (GstDecodeGroup *) group->dbin->groups->data)) {
     GST_WARNING ("Group %p is not the first group to expose", group);
-    DECODE_BIN_UNLOCK (group->dbin);
+    return FALSE;
+  }
+
+  if (group->nbdynamic) {
+    GST_WARNING ("Group %p still has %d dynamic objects, not exposing yet",
+        group, group->nbdynamic);
     return FALSE;
   }
 
   GST_LOG ("Exposing group %p", group);
 
+  /* re-order pads : video, then audio, then others */
+  group->endpads = g_list_sort (group->endpads, (GCompareFunc) sort_end_pads);
+
   /* Expose pads */
 
   for (tmp = group->endpads; tmp; tmp = next) {
@@ -1543,6 +1752,10 @@ gst_decode_group_expose (GstDecodeGroup * group)
   remove_fakesink (group->dbin);
 
   group->exposed = TRUE;
+
+  GST_LOG_OBJECT (group->dbin, "signalling no-more-pads");
+  gst_element_no_more_pads (GST_ELEMENT (group->dbin));
+
   GST_LOG_OBJECT (group->dbin, "Group %p exposed", group);
   return TRUE;
 }
@@ -1573,6 +1786,65 @@ gst_decode_group_hide (GstDecodeGroup * group)
   GROUP_MUTEX_UNLOCK (group);
 
   group->dbin->activegroup = NULL;
+  group->dbin->oldgroups = g_list_append (group->dbin->oldgroups, group);
+}
+
+static void
+deactivate_free_recursive (GstDecodeGroup * group, GstElement * element)
+{
+  GstIterator *it;
+  GstIteratorResult res;
+  gpointer point;
+
+  GST_LOG ("element:%s", GST_ELEMENT_NAME (element));
+
+  /* call on downstream elements */
+  it = gst_element_iterate_src_pads (element);
+
+restart:
+
+  while (1) {
+    res = gst_iterator_next (it, &point);
+    switch (res) {
+      case GST_ITERATOR_DONE:
+        goto done;
+      case GST_ITERATOR_RESYNC:
+        gst_iterator_resync (it);
+        goto restart;
+      case GST_ITERATOR_ERROR:
+      {
+        GST_WARNING ("Had an error while iterating source pads of element: %s",
+            GST_ELEMENT_NAME (element));
+        goto beach;
+      }
+      case GST_ITERATOR_OK:
+      {
+        GstPad *pad = GST_PAD (point);
+        GstPad *peerpad = NULL;
+
+        if ((peerpad = gst_pad_get_peer (pad))) {
+          GstObject *parent = gst_pad_get_parent (peerpad);
+
+          if (parent && GST_IS_ELEMENT (parent))
+            deactivate_free_recursive (group, GST_ELEMENT (parent));
+          if (parent)
+            gst_object_unref (parent);
+        }
+      }
+        break;
+      default:
+        break;
+    }
+  }
+
+done:
+  gst_element_set_state (element, GST_STATE_NULL);
+  gst_bin_remove (GST_BIN (group->dbin), element);
+
+beach:
+  gst_iterator_free (it);
+
+  return;
 }
 
 static void
@@ -1592,9 +1864,16 @@ gst_decode_group_free (GstDecodeGroup * group)
   g_list_free (group->endpads);
   group->endpads = NULL;
 
+  /* disconnect signal handlers on multiqueue */
+  g_signal_handler_disconnect (group->multiqueue, group->underrunsig);
+  g_signal_handler_disconnect (group->multiqueue, group->overrunsig);
+
+  /* remove all elements */
+  deactivate_free_recursive (group, group->multiqueue);
+
   GROUP_MUTEX_UNLOCK (group);
 
-  g_free (group->lock);
+  g_mutex_free (group->lock);
   g_free (group);
 }
 
@@ -1654,7 +1933,7 @@ source_pad_event_probe (GstPad * pad, GstEvent * event, GstDecodePad * dpad)
   return TRUE;
 }
 
-/** gst_decode_pad_new:
+/*gst_decode_pad_new:
  *
  * Creates a new GstDecodePad for the given pad.
  * If block is TRUE, Sets the pad blocking asynchronously
@@ -1699,13 +1978,13 @@ gst_decode_pad_new (GstDecodeGroup * group, GstPad * pad, gboolean block)
  * ::get_state .
  */
 
-static void
+static gboolean
 add_fakesink (GstDecodeBin * decode_bin)
 {
   GST_DEBUG_OBJECT (decode_bin, "Adding the fakesink");
 
   if (decode_bin->fakesink)
-    return;
+    return TRUE;
 
   decode_bin->fakesink =
       gst_element_factory_make ("fakesink", "async-fakesink");
@@ -1719,20 +1998,20 @@ add_fakesink (GstDecodeBin * decode_bin)
   if (!gst_bin_add (GST_BIN (decode_bin), decode_bin->fakesink))
     goto could_not_add;
 
-  return;
+  return TRUE;
 
   /* ERRORS */
 no_fakesink:
   {
     g_warning ("can't find fakesink element, decodebin will not work");
-    return;
+    return FALSE;
   }
 could_not_add:
   {
     g_warning ("Could not add fakesink to decodebin, decodebin will not work");
     gst_object_unref (decode_bin->fakesink);
     decode_bin->fakesink = NULL;
-    return;
+    return FALSE;
   }
 }
 
@@ -1756,7 +2035,7 @@ remove_fakesink (GstDecodeBin * decode_bin)
  * convenience functions
  *****/
 
-/** find_sink_pad
+/* find_sink_pad
  *
  * Returns the first sink pad of the given element, or NULL if it doesn't have
  * any.
@@ -1786,8 +2065,13 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
   GstDecodeBin *dbin = GST_DECODE_BIN (element);
 
   switch (transition) {
+    case GST_STATE_CHANGE_NULL_TO_READY:
+      if (dbin->typefind == NULL)
+        goto missing_typefind;
+      break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:{
-      add_fakesink (dbin);
+      if (!add_fakesink (dbin))
+        goto missing_fakesink;
       break;
     }
     default:
@@ -1799,6 +2083,22 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
   /* FIXME : put some cleanup functions here.. if needed */
 
   return ret;
+
+/* ERRORS */
+missing_typefind:
+  {
+    gst_element_post_message (element,
+        gst_missing_element_message_new (element, "typefind"));
+    GST_ELEMENT_ERROR (dbin, CORE, MISSING_PLUGIN, (NULL), ("no typefind!"));
+    return GST_STATE_CHANGE_FAILURE;
+  }
+missing_fakesink:
+  {
+    gst_element_post_message (element,
+        gst_missing_element_message_new (element, "fakesink"));
+    GST_ELEMENT_ERROR (dbin, CORE, MISSING_PLUGIN, (NULL), ("no fakesink!"));
+    return GST_STATE_CHANGE_FAILURE;
+  }
 }
 
 static gboolean
